/* ipuz-puzzle.h
 *
 * Copyright 2022 Jonathan Blandford <jrb@gnome.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <glib-object.h>
#include <json-glib/json-glib.h>

G_BEGIN_DECLS


typedef enum
{
  IPUZ_PUZZLE_FLAG_USES_EXTENSIONS = 1 << 0, /* @ uses custom extensions to the ipuz spec @ */
  IPUZ_PUZZLE_FLAG_HAS_SOLUTION    = 1 << 1, /* @ includes the solution @ */
  IPUZ_PUZZLE_FLAG_HAS_CHECKSUM    = 1 << 2, /* @ has a checksum @ */
  IPUZ_PUZZLE_FLAG_HAS_CLUES       = 1 << 3, /* @ has clues @ */
  IPUZ_PUZZLE_FLAG_HAS_SAVED       = 1 << 4, /* @ has guesses already within the ipuz file @ */
} IPuzPuzzleFlags;

typedef enum
{
  IPUZ_PUZZLE_ACROSTIC,
  IPUZ_PUZZLE_ARROWWORD,
  IPUZ_PUZZLE_BARRED,
  IPUZ_PUZZLE_CROSSWORD,
  IPUZ_PUZZLE_CRYPTIC,
  IPUZ_PUZZLE_FILIPPINE,
  IPUZ_PUZZLE_UNKNOWN,
} IPuzPuzzleKind;


#define IPUZ_TYPE_PUZZLE (ipuz_puzzle_get_type ())
G_DECLARE_DERIVABLE_TYPE (IPuzPuzzle, ipuz_puzzle, IPUZ, PUZZLE, GObject)


typedef struct _IPuzPuzzleClass IPuzPuzzleClass;
struct _IPuzPuzzleClass
{
  GObjectClass parent_class;

  /* Virtual functions */
  void                (*load_node)      (IPuzPuzzle  *puzzle,
                                         const char  *member_name,
                                         JsonNode    *node);
  void                (*post_load_node) (IPuzPuzzle  *puzzle,
                                         const char  *member_name,
                                         JsonNode    *node);
  void                (*fixup)          (IPuzPuzzle  *puzzle);
  void                (*validate)       (IPuzPuzzle  *puzzle);
  void                (*build)          (IPuzPuzzle  *puzzle,
                                         JsonBuilder *builder);
  IPuzPuzzleFlags     (*get_flags)      (IPuzPuzzle  *puzzle);
  void                (*clone)          (IPuzPuzzle  *src,
                                         IPuzPuzzle  *dest);
  const char * const *(*get_kind_str)   (IPuzPuzzle  *puzzle);
};


IPuzPuzzle     *ipuz_puzzle_new_from_json   (JsonNode       *root,
                                             GError        **error);
IPuzPuzzle     *ipuz_puzzle_new_from_file   (const char     *filename,
                                             GError        **error);
IPuzPuzzle     *ipuz_puzzle_new_from_stream (GInputStream   *stream,
                                             GCancellable   *cancellable,
                                             GError        **error);
IPuzPuzzle     *ipuz_puzzle_new_from_data   (const gchar    *data,
                                             gsize           length,
                                             GError        **error);
gboolean        ipuz_puzzle_save_to_file    (IPuzPuzzle     *puzzle,
                                             const char     *filename,
                                             GError        **error);
gboolean        ipuz_puzzle_save_to_stream  (IPuzPuzzle     *puzzle,
                                             GOutputStream  *stream,
                                             GCancellable   *cancellable,
                                             GError        **error);
gchar          *ipuz_puzzle_save_to_data    (IPuzPuzzle     *puzzle,
                                             gsize          *length);
IPuzPuzzleFlags ipuz_puzzle_get_flags       (IPuzPuzzle     *puzzle);
IPuzPuzzle     *ipuz_puzzle_deep_copy       (IPuzPuzzle     *puzzle);


G_END_DECLS
